#property copyright "Scriptong"
#property link "scriptong@mail.ru"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 DarkGray
#property indicator_color2 Lime
#property indicator_color3 Red
               

#define MAX_DAY_VOLATILITY             2000

//   
extern int       i_indBarsCount        = 5000;

//  
double g_simplyVolume[];
double g_highVolume[];
double g_lastDayMedian[];

//    
bool g_activate,                                                                                   //    
     g_init;                                                                                       //          ..
                                                                                                   // .. 
int g_volumesArray[MAX_DAY_VOLATILITY];                                                            //     

     
#define DAY_SUNDAY                     0
#define DAY_SATURDAY                   6

//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator initialization function                                                                                                                                                          |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int init()
{
   g_activate = false;                                                                             //   
   g_init = true;
   
   if (!IsTuningParametersCorrect())                                                               //      -   
      return (-1);                                 
           
   if (!BuffersBind())
      return (-1);                                                   
           
   g_activate = true;                                                                              //   
   return(0);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                          |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
bool IsTuningParametersCorrect()
{
   string name = WindowExpertName();

   if (Period() > PERIOD_H4)
   {
      Print(name, ":      ,  H4.");
      return (false);
   }

   int period = Period();
   if (period == 0)
   {
      Alert(name, ":    -  0 .  .");
      return (false);
   }
   
   if (Point == 0)
   {
      Alert(name, ":    -    . ",
                  " .");
      return (false);
   }
   
   return (true);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator deinitialization function                                                                                                                                                        |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int deinit()
{
   return(0);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                              |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
bool BuffersBind()
{
   string name = WindowExpertName();

   //     
   if (!SetIndexBuffer(0, g_simplyVolume)             ||
       !SetIndexBuffer(1, g_highVolume)               ||
       !SetIndexBuffer(2, g_lastDayMedian))
   {
      Alert(name, ":      .  ", GetLastError());
      return (false);
   }

   //    
   for (int i = 0; i < 2; i++)   
      SetIndexStyle(i, DRAW_HISTOGRAM);
      
   SetIndexStyle(2, DRAW_LINE);
   
   SetIndexLabel(0, "Volume");
   SetIndexLabel(1, "High volume");
   SetIndexLabel(2, "Last day median");
   
   return (true);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                     |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void BuffersInitialize()
{
   ArrayInitialize(g_simplyVolume, EMPTY_VALUE);
   ArrayInitialize(g_highVolume, EMPTY_VALUE);
   ArrayInitialize(g_lastDayMedian, EMPTY_VALUE);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|   ,                                                                                                                                 |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int GetRecalcIndex(int& total)
{
   //      
   static int lastBarsCnt;                                                
   if (g_init)                                     
   {                                               
      lastBarsCnt = 0;
      g_init = false;
   }
   total = Bars - 2;                                                                               //    ,       
                                                   
   //          ?
   if (i_indBarsCount > 0 && i_indBarsCount < total)
      total = i_indBarsCount;                      
                                                   
   //       , . .          ,     ,      
   if (lastBarsCnt < Bars - 1)                     
   {                                               
      lastBarsCnt = Bars;
      BuffersInitialize();
      return (total);
   }
   
   //   .            ,    
   int newBarsCnt = Bars - lastBarsCnt;
   lastBarsCnt = Bars;
   return (newBarsCnt);                            
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                   |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
double GetMedianValue(int& array[], int cnt)
{
   ArraySort(array, cnt);
   
   //                . 
   //       -      
   int halfElement = cnt / 2;
   if (MathMod(cnt, 2) == 0)
      return ((array[halfElement] + array[halfElement - 1]) / 2);

   return (array[halfElement]);   
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                    |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int GetMedianOfPrevDayVolume(int index)
{
   ArrayInitialize(g_volumesArray, 0);
   int arraySize = 0;
   for (int i = index; i < Bars; i++)
   {
      int dayOfWeek = TimeDayOfWeek(Time[i]);
      if (dayOfWeek == DAY_SUNDAY || dayOfWeek == DAY_SATURDAY)
         continue;
   
      g_volumesArray[arraySize] = Volume[i];
      arraySize++;
      if (arraySize >= MAX_DAY_VOLATILITY)
         break;

      if (TimeDayOfYear(Time[i]) != TimeDayOfYear(Time[i + 1]))
         break;
   }
   
   return (GetMedianValue(g_volumesArray, arraySize));
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                       |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void ProcessOneCandle(int index)
{
   //        
   static int maxVolume;
   if (TimeDayOfYear(Time[index]) != TimeDayOfYear(Time[index + 1]))
   {
      g_lastDayMedian[index] = GetMedianOfPrevDayVolume(index + 1);
      maxVolume = g_lastDayMedian[index];
   }
   else
      g_lastDayMedian[index] = g_lastDayMedian[index + 1];   

   //       
   if (Volume[index] <= maxVolume)
   {
      g_simplyVolume[index] = Volume[index];
      g_highVolume[index] = EMPTY_VALUE;
      return;
   }

   maxVolume = Volume[index];
   g_simplyVolume[index] = EMPTY_VALUE;
   g_highVolume[index] = maxVolume;
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                        |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void ShowIndicatorData(int limit, int total)
{
   for (int i = limit; i >= 0; i--)
      ProcessOneCandle(i);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator iteration function                                                                                                                                                               |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int start()
{
   if (!g_activate)                                                                                //     ,     
      return (0);                                 
                                                   
   int total;   
   int limit = GetRecalcIndex(total);                                                              //     ?

   ShowIndicatorData(limit, total);                                                                //   
   
   return(0);
}